@@ -1,11 +1,13 @@
 #!/usr/bin/env python
-
+import copy
+import ctypes
 import datetime
 import gzip
 import hashlib
 import os
 import re
 import shutil
+import stat
 import subprocess
 import sys
 import tempfile
@@ -13,6 +15,8 @@ import time
 from io import BytesIO
 from xml.sax.saxutils import escape
 
+from univers.rpm import compare_rpm_versions
+
 import rpmfile
 import storage
 
@@ -338,6 +342,30 @@ def parse_primary(data):
                                     'ver': obsoletes_ver,
                                     'flags': obsoletes_flags}
 
+        # conflicts
+
+        conflicts = fmt.find('rpm:conflicts', namespaces)
+        if conflicts is None:
+            conflicts = []
+
+        conflicts_dict = {}
+
+        for entry in conflicts:
+            conflicts_name = entry.attrib['name']
+            conflicts_epoch = entry.attrib.get('epoch', None)
+            conflicts_rel = entry.attrib.get('rel', None)
+            conflicts_ver = entry.attrib.get('ver', None)
+            conflicts_flags = entry.attrib.get('flags', None)
+
+            nerv = (conflicts_name, conflicts_epoch,
+                    conflicts_rel, conflicts_ver)
+
+            conflicts_dict[nerv] = {'name': conflicts_name,
+                                    'epoch': conflicts_epoch,
+                                    'rel': conflicts_rel,
+                                    'ver': conflicts_ver,
+                                    'flags': conflicts_flags}
+
         # files
         files = []
         for node in fmt.findall('primary:file', namespaces):
@@ -359,6 +387,7 @@ def parse_primary(data):
                        'provides': provides_dict,
                        'requires': requires_dict,
                        'obsoletes': obsoletes_dict,
+                       'conflicts': conflicts_dict,
                        'files': files}
 
         package = {
@@ -434,6 +463,97 @@ def parse_other(data):
     return packages
 
 
+def add_requires_entry(res, requires):
+    entry = ['name="%s"' % requires['name']]
+    for component in ['flags', 'epoch', 'ver', 'rel', 'pre']:
+        if requires[component] is not None:
+            entry.append('%s="%s"' % (component, requires[component]))
+
+    res += '      <rpm:entry ' + escape(' '.join(entry)) + '/>\n'
+    return res
+
+
+def compare_dependency(dep1: str, dep2: str) -> int:
+    """
+    Compares two dependencies by name
+    NOTE: The function assumes first parts must be same!
+    libc.so.6() < libc.so.6(GLIBC_2.3.4)(64 bit) < libc.so.6(GLIBC_2.4)
+    @return values:
+        0 - same
+        1 - first is bigger
+        2 - second is bigger
+       -1 - error
+    """
+    if dep1 == dep2:
+        return 0
+
+    ver1_idx = dep1.find('(')
+    ver2_idx = dep2.find('(')
+
+    # There is no '('
+    if ver1_idx == -1 and ver2_idx == -1:
+        return 0
+    if ver1_idx == -1:
+        return 2
+    if ver2_idx == -1:
+        return 1
+
+    ver1 = dep1[ver1_idx:]
+    ver2 = dep2[ver2_idx:]
+
+    ver1_e_idx = ver1.find(')')
+    ver2_e_idx = ver2.find(')')
+
+    # There is no ')'
+    if ver1_e_idx == -1 and ver2_e_idx == -1:
+        return -1
+    if ver1_e_idx == -1:
+        return 2
+    if ver2_e_idx == -1:
+        return 1
+
+    ver1_e = ver1[ver1_e_idx:]
+    ver2_e = ver2[ver2_e_idx:]
+
+    # Go to char next to '('
+    ver1 = dep1[ver1_idx + 1:]  # libc.so.6(...
+    ver2 = dep2[ver2_idx + 1:]  # verX     ^
+
+    # If parentheses have no content - libc.so.6()... == libc.so.6()...
+    if ver1 == ver1_e and ver2 == ver2_e:
+        return 0
+    if ver1 == ver1_e:
+        return 2
+    if ver2 == ver2_e:
+        return 1
+
+    # Go to first number
+    ver1_num_idx = None
+    for i in range(len(ver1)):
+        if ver1[i].isnumeric():
+            ver1_num_idx = i
+            break
+
+    ver2_num_idx = None
+    for i in range(len(ver2)):
+        if ver2[i].isnumeric():
+            ver2_num_idx = i
+            break
+
+    ver1 = ver1[ver1_num_idx:]
+    ver2 = ver2[ver2_num_idx:]
+
+    # Get version string
+    ver1 = ver1[:-len(ver1_e)]
+    ver2 = ver2[:-len(ver2_e)]
+
+    ret1 = compare_rpm_versions(ver1, ver2)
+    if ret1 == -1:
+        ret1 = 2
+
+    return ret1
+
+
 def dump_primary(primary):
     res = ""
 
@@ -522,14 +642,26 @@ def dump_primary(primary):
 
         res += '    <rpm:requires>\n'
 
+        libc_require_highest = None
         for key in sorted(fmt['requires'], key=sort_key):
             requires = fmt['requires'][key]
-            entry = ['name="%s"' % requires['name']]
-            for component in ['flags', 'epoch', 'ver', 'rel', 'pre']:
-                if requires[component] is not None:
-                    entry.append('%s="%s"' % (component, requires[component]))
+            requires_name = requires['name']
 
-            res += '      <rpm:entry ' + escape(' '.join(entry)) + '/>\n'
+            # libc.so.6 filtering
+            # Require name goes in alphabetical order.
+            # This means if there's a name starting with libc.so.6,
+            # consecutive names will be the same
+            if requires_name.startswith('libc.so.6'):
+                if not libc_require_highest:
+                    libc_require_highest = requires
+                else:
+                    if compare_dependency(libc_require_highest['name'], requires_name) == 2:
+                        libc_require_highest = requires
+                continue
+            if libc_require_highest:
+                res = add_requires_entry(res, libc_require_highest)
+                libc_require_highest = None
+            res = add_requires_entry(res, requires)
 
         res += '    </rpm:requires>\n'
 
@@ -546,6 +678,35 @@ def dump_primary(primary):
 
         res += '    </rpm:obsoletes>\n'
 
+        if fmt['conflicts']:
+
+            res += '    <rpm:conflicts>\n'
+
+            for key in sorted(fmt['conflicts'], key=sort_key):
+                conflicts = fmt['conflicts'][key]
+                entry = ['name="%s"' % conflicts['name']]
+                for component in ['flags', 'epoch', 'ver', 'rel']:
+                    if conflicts[component] is not None:
+                        entry.append('%s="%s"' % (component, conflicts[component]))
+
+                res += '      <rpm:entry ' + escape(' '.join(entry)) + '/>\n'
+
+            res += '    </rpm:conflicts>\n'
+
+        primary_dirs_files = []
+        for file in fmt['files']:
+            if is_primary_file(file['name']):
+                primary_dirs_files.append(file)
+
+        if primary_dirs_files:
+            for primary_dir_file in primary_dirs_files:
+                primary_dir_file_t = primary_dir_file["type"]
+                primary_dir_file_n = primary_dir_file["name"]
+                if primary_dir_file_t == 'dir':
+                    res += f'  <file type="{primary_dir_file_t}">{primary_dir_file_n}</file>\n'
+                elif primary_dir_file_t == 'file':
+                    res += f'  <file>{primary_dir_file_n}</file>\n'
+
         res += '  </format>\n'
         res += '</package>\n'
 
@@ -554,6 +715,17 @@ def dump_primary(primary):
     return res
 
 
+def is_primary_file(file_name: str) -> bool:
+    """Check if the filename should be listed in primary.xml."""
+    if file_name.startswith('/etc/'):
+        return True
+    elif file_name == '/usr/lib/sendmail':
+        return True
+    elif 'bin/' in file_name:
+        return True
+    return False
+
+
 def parse_ver_str(ver_str):
     if not ver_str:
         return (None, None, None)
@@ -690,6 +862,36 @@ def get_arch_from_header(header):
         return get_with_decode(header, 'ARCH', None)
 
 
+def _get_files(header):
+    dirnames = header.get('DIRNAMES', [])
+    if not isinstance(dirnames, list):
+        dirnames = [dirnames]
+    basenames = header.get('BASENAMES', [])
+    if not isinstance(basenames, list):
+        basenames = [basenames]
+    dirindexes = header.get('DIRINDEXES', [])
+    if not isinstance(dirindexes, list):
+        dirindexes = [dirindexes]
+    filemodes = header.get('FILEMODES', [])
+    if not isinstance(filemodes, list):
+        filemodes = [filemodes]
+
+    files = []
+
+    # Represent integer as a C uint16 type value.
+    filemodes = [ctypes.c_uint16(filemode).value for filemode in filemodes]
+
+    for entry in zip(basenames, dirindexes, filemodes):
+        filename = entry[0].decode('utf-8')
+        dirname = dirnames[entry[1]].decode('utf-8')
+        if stat.S_ISDIR(entry[2]):
+            files.append({'name': dirname + filename, 'type': 'dir'})
+        elif stat.S_ISREG(entry[2]) or stat.S_ISLNK(entry[2]):
+            files.append({'name': dirname + filename, 'type': 'file'})
+
+    return files
+
+
 def header_to_filelists(header, sha256):
     pkgid = sha256
     name = get_with_decode(header, 'NAME', None)
@@ -699,39 +901,7 @@ def header_to_filelists(header, sha256):
     ver = get_with_decode(header, 'VERSION', None)
     version = {'ver': ver, 'rel': rel, 'epoch': epoch}
 
-    dirnames = header.get('DIRNAMES', [])
-    if not isinstance(dirnames, list):
-        dirnames = [dirnames]
-    classdict = header.get('CLASSDICT', [])
-    if not isinstance(classdict, list):
-        classdict = [classdict]
-    basenames = header.get('BASENAMES', [])
-    if not isinstance(basenames, list):
-        basenames = [basenames]
-    dirindexes = header.get('DIRINDEXES', [])
-    if not isinstance(dirindexes, list):
-        dirindexes = [dirindexes]
-    fileclasses = header.get('FILECLASS', [])
-    if not isinstance(fileclasses, list):
-        fileclasses = [fileclasses]
-
-    files = []
-
-    for entry in zip(basenames, dirindexes, fileclasses):
-        filename = entry[0].decode('utf-8')
-        dirname = dirnames[entry[1]].decode('utf-8')
-
-        fileclass = classdict[entry[2]]
-
-        filetype = "file"
-
-        if fileclass == "directory":
-            filetype = "dir"
-
-        files.append({'name': dirname + filename, 'type': filetype})
-
-    for dirname in dirnames:
-        files.append({'name': dirname.decode('utf-8'), 'type': 'dir'})
+    files = _get_files(header)
 
     package = {'pkgid': pkgid, 'name': name, 'arch': arch,
                'version': version, 'files': files}
@@ -750,8 +920,21 @@ def header_to_primary(
         size):
     name = get_with_decode(header, 'NAME', None)
     arch = get_arch_from_header(header)
-    summary = get_with_decode(header, 'SUMMARY')
-    description = get_with_decode(header, 'DESCRIPTION')
+
+    try:
+        summary = get_with_decode(header, 'SUMMARY')
+    except UnicodeDecodeError:
+        summary = get_with_decode(
+            header, 'SUMMARY', encoding='latin-1'
+        ).encode('utf-8').decode('utf-8')
+
+    try:
+        description = get_with_decode(header, 'DESCRIPTION')
+    except UnicodeDecodeError:
+        description = get_with_decode(
+            header, 'DESCRIPTION', encoding='latin-1'
+        ).encode('utf-8').decode('utf-8')
+
     packager = get_with_decode(header, 'PACKAGER', None)
     build_time = header.get('BUILDTIME', '')
     url = get_with_decode(header, 'URL')
@@ -798,6 +981,10 @@ def header_to_primary(
                                'ver': provides_ver,
                                'flags': provides_flags}
 
+    # files
+
+    files = _get_files(header)
+
     # requires
 
     requires_dict = {}
@@ -823,12 +1010,25 @@ def header_to_primary(
 
         nerv = (requires_name, requires_epoch, requires_rel, requires_ver)
 
-        requires_dict[nerv] = {'name': requires_name,
-                               'epoch': requires_epoch,
-                               'rel': requires_rel,
-                               'ver': requires_ver,
-                               'flags': requires_flags,
-                               "pre": pre}
+        nerv_entry = {'name': requires_name,
+                      'epoch': requires_epoch,
+                      'rel': requires_rel,
+                      'ver': requires_ver,
+                      'flags': requires_flags
+                      }
+
+        # Skip files which are provided.
+        if nerv in provides_dict:
+            if nerv_entry == provides_dict[nerv]:
+                continue
+
+        # Skip package primary files.
+        full_filenames = [f['name'] for f in files if f['type'] == 'file']
+        if requires_name in full_filenames and is_primary_file(requires_name):
+            continue
+
+        nerv_entry['pre'] = pre
+        requires_dict[nerv] = nerv_entry
 
     # obsoletes
 
@@ -854,25 +1054,29 @@ def header_to_primary(
                                 'ver': obsoletes_ver,
                                 'flags': obsoletes_flags}
 
-    # files
-    dirnames = header.get('DIRNAMES', [])
-    if not isinstance(dirnames, list):
-        dirnames = [dirnames]
-    basenames = header.get('BASENAMES', [])
-    if not isinstance(basenames, list):
-        basenames = [basenames]
-    dirindexes = header.get('DIRINDEXES', [])
-    if not isinstance(dirindexes, list):
-        dirindexes = [dirindexes]
+    # conflicts
 
-    files = []
-    for entry in zip(basenames, dirindexes):
-        filename = entry[0].decode('utf-8')
-        dirname = dirnames[entry[1]].decode('utf-8')
-        files.append({'name': dirname + filename, 'type': 'file'})
+    conflicts_dict = {}
+    conflictname = header.get('CONFLICTNAME', [])
+    conflictversion = header.get('CONFLICTVERSION', [])
+    conflictflags = header.get('CONFLICTFLAGS', [])
 
-    for dirname in dirnames:
-        files.append({'name': dirname.decode('utf-8'), 'type': 'dir'})
+    if not isinstance(conflictflags, list):
+        conflictflags = [conflictflags]
+
+    for entry in zip(conflictname, conflictversion, conflictflags):
+        conflicts_name = entry[0].decode('utf-8')
+        conflicts_epoch, conflicts_ver, conflicts_rel = \
+            parse_ver_str(entry[1].decode('utf-8'))
+        conflicts_flags = rpmfile.flags_to_str(entry[2])
+
+        nerv = (conflicts_name, conflicts_epoch, conflicts_rel, conflicts_ver)
+
+        conflicts_dict[nerv] = {'name': conflicts_name,
+                                'epoch': conflicts_epoch,
+                                'rel': conflicts_rel,
+                                'ver': conflicts_ver,
+                                'flags': conflicts_flags}
 
     # result package
     format_dict = {'license': format_license,
@@ -885,6 +1089,7 @@ def header_to_primary(
                    'provides': provides_dict,
                    'requires': requires_dict,
                    'obsoletes': obsoletes_dict,
+                   'conflicts': conflicts_dict,
                    'files': files}
 
     package = {
@@ -1064,10 +1269,27 @@ def update_repo(storage, sign, tempdir, force=False):
         existing_files.add((file_path, mtime))
 
     files_to_add = existing_files - recorded_files
+    files_to_delete = recorded_files - existing_files
     # List of packages that can't be added to the index
     # (some problems encountered during processing).
     malformed_list = []
 
+    if files_to_delete:
+        stale_primary = {}
+        copied_primary = copy.deepcopy(primary)
+        for file_to_delete in files_to_delete:
+            for primary_nerv, primary_value in list(copied_primary.items())[:]:
+                if primary_value['location'] == file_to_delete[0]:
+                    stale_primary[primary_nerv] = primary_value
+                    del copied_primary[primary_nerv]
+                    break
+
+        for stale_nerv, stale_value in stale_primary.items():
+            del primary[stale_nerv]
+            del filelists[stale_nerv]
+            del others[stale_nerv]
+            print(f"Deleting: '{stale_value['location']}'")
+
     for file_to_add in files_to_add:
         file_path = file_to_add[0]
         mtime = file_to_add[1]
@@ -1160,7 +1382,7 @@ def update_repo(storage, sign, tempdir, force=False):
 def main():
     stor = storage.FilesystemStorage(sys.argv[1])
 
-    update_repo(stor)
+    update_repo(stor, sign=False, tempdir='/tmp')
 
 
 if __name__ == '__main__':
